package com.bananasound.server.config.security;

import com.bananasound.server.commons.exception.ResourceNotFoundException;
import com.bananasound.server.modules.user.entity.UserEntity;
import com.bananasound.server.modules.user.repository.UserRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class SecurityUserDetailsService implements UserDetailsService {
  @Autowired
  UserRepository userRepository;

  @Override
  @Transactional
  public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {

    // Let people login with either username or email
    UserEntity user = userRepository.findByUsername(username)
        .orElseThrow(() -> new UsernameNotFoundException("User not found with username : " + username));

    return SecurityUserDetails.create(user);
  }

  @Transactional
  public UserDetails loadUserById(Long id) {
    UserEntity user = userRepository.findById(id).orElseThrow(() -> new ResourceNotFoundException("User", "id", id));

    return SecurityUserDetails.create(user);
  }
}
