package com.bananasound.server.modules.user.entity;

import java.util.Set;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.bananasound.server.commons.data.entity.DateAuditEntity;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.Builder.Default;
import lombok.experimental.SuperBuilder;

@Entity
@Table(name = "users", uniqueConstraints = { @UniqueConstraint(columnNames = { "username" }),
    @UniqueConstraint(columnNames = { "email" }) })
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class UserEntity extends DateAuditEntity {
  private static final long serialVersionUID = 8891163223262220481L;

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Long id;

  @Basic
  @Column(name = "username", unique = true, nullable = false, length = 150)
  private String username;

  @Basic
  @Column(name = "password", nullable = false, length = 100)
  @JsonIgnore
  private String password;

  @Basic
  @Column(name = "nickname", nullable = false, length = 150)
  private String nickname;

  @Basic
  @Column(name = "email", nullable = false, length = 100)
  private String email;

  @Basic
  @Column(name = "block", nullable = false)
  @Default
  private Boolean block = false;

  @Basic
  @Column(name = "send_email", nullable = false)
  @Default
  private Boolean sendEmail = true;

  @ManyToMany(fetch = FetchType.EAGER)
  @JoinTable(name = "user_roles", joinColumns = @JoinColumn(name = "user_id"), inverseJoinColumns = @JoinColumn(name = "role_id"))
  private Set<RoleEntity> roles;
}
